#include "toolbox.hpp"
#include "config.hpp"
#include "toolbox_unit_test.hpp"
#include "ViolaJones.hpp"

#if GPU_BOOSTED
#include "gpu_unit_test.hpp"
#endif

int32_t main(void){
	setlocale(LC_NUMERIC, ""); // Allow proper number display

	const std::chrono::system_clock::time_point unit_timestamp = perf_counter_ns();
	const std::array<int32_t, 3> unit_gaps = { 27, -18, 29 };
	header(unit_gaps, { "Unit testing", "Time spent (ns)", "Formatted time spent" });
#if GPU_BOOSTED
	benchmark_function_void("Testing GPU capabilities 1D", unit_gaps[0], test_working, 50000);
	benchmark_function_void("Testing GPU capabilities 2D", unit_gaps[0], test_working_2d, 200, 500);
	benchmark_function_void("Testing GPU capabilities 3D", unit_gaps[0], test_working_3d, 30, 40, 500);
#endif
	benchmark_function_void("Testing format_time", unit_gaps[0], format_time_test);
	benchmark_function_void("Testing format_time_ns", unit_gaps[0], format_time_ns_test);
	benchmark_function_void("Testing format_byte_size", unit_gaps[0], format_byte_size_test);
	benchmark_function_void("Testing thousand_sep", unit_gaps[0], thousand_sep_test);
	const long long time_spent = duration_ns(perf_counter_ns() - unit_timestamp);
	formatted_line(unit_gaps, "├", "┼", "─", "┤");
	formatted_row(unit_gaps, { "Unit testing summary", thousand_sep(time_spent).c_str(), format_time_ns(time_spent).c_str() });
	footer(unit_gaps);

	return EXIT_SUCCESS;
}
